@@ -88,8 +88,7 @@ parseCommandLine(int argc, const char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 --------------------------------------------------------------------------*/
-    optEntry * option_def;
-        /* Instructions to pm_optParseOptions3 on how to parse our options. */
+    optEntry * option_def;   /* Used by OPTENT3 */
     optStruct3 opt;
 
     unsigned int imageSpec;
@@ -121,7 +120,7 @@ parseCommandLine(int argc, const char ** argv,
 
     /* Set some defaults the lazy way (using multiple setting of variables) */
 
-    pm_optParseOptions3(&argc, (char**)argv, opt, sizeof(opt), 0);
+    pm_optParseOptions4(&argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (imageSpec) {
@@ -361,7 +360,6 @@ readCard(FILE * const ifP,
 
 
 
-
 static void
 processNaxisN(unsigned int   const n,
               int            const value,
@@ -397,7 +395,7 @@ readFitsHeader(FILE *               const ifP,
 
     bool gotEmpty, gotSimple, gotNaxis, gotN1, gotN2, gotN3, gotBitpix, gotEnd;
 
-    gotEmpty  = false;  /* initial value */    
+    gotEmpty  = false;  /* initial value */
     gotSimple = false;  /* initial value */
     gotNaxis  = false;  /* initial value */
     gotN1     = false;  /* initial value */
@@ -463,7 +461,7 @@ readFitsHeader(FILE *               const ifP,
                 gotEnd = true;
                 if (gotEmpty == true)
                     pm_message("Blank card(s) were encountered before "
-			       "END in header");
+                               "END in header");
             }
         }
     }
